@@ -1,3 +1,8 @@
+import os
+
+if os.environ.get("PYTORCH_ENABLE_MPS_FALLBACK") is None:
+    os.environ["PYTORCH_ENABLE_MPS_FALLBACK"] = "1"
+
 from inference_exp.entities import ColorFormat
 from inference_exp.models.auto_loaders.core import AutoModel
 from inference_exp.models.base.classification import (
